# bmstu-schedule-parser
# Copyright (C) 2018 BMSTU Schedule (George Gabolaev)

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import requests
import re
import sys

from bmstu_schedule.logger import AwesomeLogger
from bmstu_schedule import configs

def group_code_formatter(group_url):
    try:
        return group_url.text.lstrip()[:11].rstrip()
    except AttributeError:
        AwesomeLogger.shit(
            'There is no schedule for the group you specified.')
        sys.exit(-1)


def get_urls(group_code, outdir, soup):

    if group_code != configs.ALL_GROUPS_PARAM:
        group_url_button = soup.find(
            'a', {
                'class': re.compile('.*btn btn-sm btn-default text-nowrap.*')
            }, text=re.compile(r'.*\ {}.*'.format(group_code))
        )
        valid_group_code = group_code_formatter(group_url_button)
        AwesomeLogger.info('{} group found'.format(valid_group_code))
        yield (
            valid_group_code,
            configs.MAIN_URL + group_url_button.attrs['href']
        )
    else:
        yield from unload_all_groups(soup, outdir)


def unload_all_groups(soup, outdir):
    all_urls = soup.find_all(
        'a', 'btn btn-sm btn-default text-nowrap')
    urls_count = len(all_urls)

    mapping = []
    for url_id, group_url_button in enumerate(all_urls):
        try:
            valid_group_code = group_code_formatter(group_url_button)
            AwesomeLogger.info('Processing {} | {}/{} | [{}%]'.format(
                valid_group_code,
                url_id,
                urls_count,
                round(url_id / urls_count * 100, 2)
            ))

            url = configs.MAIN_URL + group_url_button['href']

            yield (
                valid_group_code,
                url,
            )

            mapping.append({
                "group": valid_group_code,
                "url": url
            })
        except Exception as ex:
            AwesomeLogger.shit((ex, url_id, group_url_button))

    with open(outdir + '/mapping.json', 'w', encoding='utf-8') as mapping_file:
        mapping_file.write(json.dumps(mapping, ensure_ascii=False))



# bmstu-schedule-parser
# Copyright (C) 2018 BMSTU Schedule (George Gabolaev)

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# time-section
DATE_FORMAT = '%d-%m-%Y'
API_URL = 'http://142.93.174.191/start_date/'

# iCal section
STABLE = 17
PERIODIC = 9
ICAL_HEADER = 'BEGIN:VCALENDAR\nVERSION:2.0\nCALSCALE:GREGORIAN'
ICAL_BODY = \
    '''
    BEGIN:VEVENT
    SUMMARY:{summary}
    DTSTART:{startDate}T{startTime}
    DTEND:{endDate}T{endTime}
    RRULE:FREQ=WEEKLY;INTERVAL={interval};COUNT={count};
    LOCATION:{auditorium}
    DESCRIPTION:{professor}
    END:VEVENT'''
ICAL_BOTTOM = '\nEND:VCALENDAR'

# bmstu
MAIN_URL = 'https://students.bmstu.ru'
GROUPS_LIST_URL = '/schedule/list'


# logic
ALL_GROUPS_PARAM = 'all'

# physical culture
PC_LESSON_KEYREGEX = r'.*спорту.*'
PC_LESSONS_TIMES_MAPPING = {
    '083000': ('081500','094500'),
    '101500': ('100000','113000'),
    '120000': ('122000','135000'),
    '135000': ('140500','153500'),
    '154000': ('155000','172000'),
    '172500': ('172500','185500'),
}


# bmstu-schedule-parser
# Copyright (C) 2018 BMSTU Schedule (George Gabolaev)

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import argparse as ap
from datetime import datetime as dt
import requests
from bmstu_schedule import configs, AwesomeLogger, run


def setup_parser():
    parser = ap.ArgumentParser(
        description='Bauman Moscow State Technical University ' +
        'EU Schedule iCal parser')

    parser.add_argument(
        '-s',
        '--semester_first_monday',
        default='',
        type=date_parser,
        help='Semester first week monday date',
    )

    parser.add_argument(
        '-o',
        '--outdir',
        default='.',
        type=str,
        help='Outdir path',
    )

    parser.add_argument(
        'group',
        type=group_code_handler,
        help='Group code'
    )

    return parser


def group_code_handler(group_code):
    return group_code.upper() if group_code != configs.ALL_GROUPS_PARAM else group_code


def get_api_date():
    AwesomeLogger.info('Fetching {}'.format(configs.API_URL))
    r = requests.get(url=configs.API_URL)
    return r.json()['semester_start_date']


def date_parser(date):
    try:
        return dt.strptime(date or get_api_date(), configs.DATE_FORMAT)
    except Exception:
        msg = 'date has incorrect format. It must be: "dd-mm-yyyy", got: "{}"'.format(
            date
        )
        raise ap.ArgumentTypeError(msg)


def main():
    args = setup_parser().parse_args()
    AwesomeLogger.info(
        "Semester start date: {}".format(args.semester_first_monday.date())
        )
    try:
        run(args.group, args.semester_first_monday, args.outdir)
    except ConnectionError as ex:
        AwesomeLogger.shit(ex)


if __name__ == "__main__":
    main()



# bmstu-schedule-parser
# Copyright (C) 2018 BMSTU Schedule (George Gabolaev)

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import textwrap
import re

from datetime import timedelta as tdelta
from bmstu_schedule import configs

class Subject:
    semester_start_date = None

    def __init__(self, info, subject_day_index, weeks_interval, denominator):
        self.type, self.name, self.auditorium, self.professor = info

        time_shift = denominator * 7 + subject_day_index
        self.start_date = (
            self.semester_start_date + tdelta(days=time_shift)
        ).strftime('%Y%m%d')
        self.weeks_interval = weeks_interval


class Lesson:

    def __init__(self, timing, subjects):
        self.start_time, self.end_time = map(
            lambda string: string.replace(':', '') + '00',
            timing.split(' - ')
        )
        self.subjects = subjects

    def write_ics_to_file(self, file):
        for subject in self.subjects:
            if re.match(configs.PC_LESSON_KEYREGEX, subject.name):
                beginning, ending = configs.PC_LESSONS_TIMES_MAPPING[self.start_time]
            else:
                beginning, ending = self.start_time, self.end_time

            event = configs.ICAL_BODY.format(
                summary='{} {}'.format(subject.type or '', subject.name),
                startDate=subject.start_date,
                startTime=beginning,
                endDate=subject.start_date,
                endTime=ending,
                interval=subject.weeks_interval,
                count=configs.STABLE if subject.weeks_interval == 1
                else configs.PERIODIC,
                auditorium=subject.auditorium or '',
                professor=subject.professor or ''
            )
            file.write(textwrap.dedent(event))


def parse_row(cells, day_number, file):
    if len(set(cell for cell in cells[3:5])) > 1:
        subjects = []
        timing = cells[1].string

        for c in range(3, 5):
            try:
                subjects.append(
                    Subject(
                        (cells[c].contents[i].string for i in range(0, 7, 2)),
                        day_number,
                        weeks_interval=(
                            2 if cells[3].attrs != {
                                'colspan': '2'
                            } else 1
                        ),
                        denominator=(c == 4)
                    )
                )
            except (IndexError, AttributeError):
                pass

        Lesson(timing, subjects).write_ics_to_file(file)





# bmstu-schedule-parser
# Copyright (C) 2018 BMSTU Schedule (George Gabolaev)

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import textwrap
import requests
from bs4 import BeautifulSoup as bsoup
from bmstu_schedule import configs
from bmstu_schedule.logger import AwesomeLogger
from bmstu_schedule.group_page_search import get_urls, unload_all_groups
from bmstu_schedule.day import parse_row, Subject


def run(group_code, semester_first_monday, outdir):
    Subject.semester_start_date = semester_first_monday

    AwesomeLogger.info('Going to schedules list page')
    list_page_response = requests.get(configs.MAIN_URL + configs.GROUPS_LIST_URL)
    AwesomeLogger.info('Parsing your group(s) url(s)')
    soup = bsoup(list_page_response.content, 'lxml')

    for valid_group_code, url in get_urls(group_code, outdir, soup):
        AwesomeLogger.info('Going to your group schedule page')
        page_html = requests.get(url)

        AwesomeLogger.info('Parsing your schedule')
        soup = bsoup(page_html.content, 'lxml')

        try:
            filename = '{}/{}.ics'.format(outdir, valid_group_code)
            with open(filename, 'w', encoding='u8') as ics:
                ics.writelines(textwrap.dedent(configs.ICAL_HEADER))

                for dID, day in enumerate(soup.select('div.col-md-6.hidden-xs')):
                    day_table = day.contents[1]
                    rows = day_table.findAll('tr')
                    for row in rows[2:]:
                        parse_row(row.contents, dID, ics)

                ics.write(textwrap.dedent(configs.ICAL_BOTTOM))
        except FileNotFoundError:
            AwesomeLogger.shit(
                'No such file or directory: {}'.format(outdir))
            return

        AwesomeLogger.info('Done!')
        AwesomeLogger.info('File saved at {}/{}.ics'.format(outdir, valid_group_code))
        AwesomeLogger.info('Now you can import it.')




# bmstu-schedule-parser
# Copyright (C) 2018 BMSTU Schedule (George Gabolaev)

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from setuptools import setup

setup(
    name='bmstu-schedule',
    version='1.4.2',
    author='George Gabolaev',
    author_email='gabolaev98@gmail.com',
    url='https://github.com/BMSTU-Schedule/bmstu-schedule-parser',
    license='AGPLv3',
    python_requires='>=3.5',
    long_description_content_type='text/markdown',
    description='BMSTU EU Schedule iCal parser',
    long_description=open('README.md').read(),
    packages=['bmstu_schedule', 'bmstu_schedule.configs'],
    install_requires=[
        'lxml',
        'bs4',
        'requests',
        'termcolor'
    ],
    entry_points={
        'console_scripts': [
            'bmstu-schedule = bmstu_schedule.__main__:main'
        ]
    },
)
